# frozen_string_literal: true

module MergeRequests
  class UpdateService < MergeRequests::BaseService
    extend ::Gitlab::Utils::Override

    def initialize(project, user = nil, params = {})
      super

      @target_branch_was_deleted = @params.delete(:target_branch_was_deleted)
    end

    def execute(merge_request)
      # Contrary to GitLab, Heptapod does allow change of source branch after
      # merge request was created, but still not of source/target projects
      params.delete(:source_project_id)
      params.delete(:target_project_id)

      if merge_request.closed_or_merged_without_fork?
        params.delete(:target_branch)
        params.delete(:force_remove_source_branch)
      end

      update_task_event(merge_request) || update(merge_request)
    end

    # TODO: Cyclomatic complexity and Abc size offenses appeared with upstream
    #       13.9 merge, not the proper time to refactor that.
    # rubocop:disable Metrics/CyclomaticComplexity
    # rubocop:disable Metrics/AbcSize
    def handle_changes(merge_request, options)
      old_associations = options.fetch(:old_associations, {})
      old_labels = old_associations.fetch(:labels, [])
      old_mentioned_users = old_associations.fetch(:mentioned_users, [])
      old_assignees = old_associations.fetch(:assignees, [])
      old_reviewers = old_associations.fetch(:reviewers, [])

      if has_changes?(merge_request, old_labels: old_labels, old_assignees: old_assignees, old_reviewers: old_reviewers)
        todo_service.resolve_todos_for_target(merge_request, current_user)
      end

      if merge_request.previous_changes.include?('title') ||
          merge_request.previous_changes.include?('description')
        todo_service.update_merge_request(merge_request, current_user, old_mentioned_users)
      end

      branch_changes = []
      branch_changes << 'target' if merge_request.previous_changes.include?('target_branch')
      branch_changes << 'source' if merge_request.previous_changes.include?('source_branch')
      branch_changes.each do |type|
        type_branch = type + '_branch'
        # rubocop:disable GitlabSecurity/PublicSend
        create_branch_change_note(merge_request, type,
                                  type == 'target' && target_branch_was_deleted ? 'delete' : 'update',
                                  merge_request.previous_changes[type_branch].first,
                                  merge_request.public_send(type_branch))
        # rubocop:enable GitlabSecurity/PublicSend
      end

      handle_assignees_change(merge_request, old_assignees) if merge_request.assignees != old_assignees

      handle_reviewers_change(merge_request, old_reviewers) if merge_request.reviewers != old_reviewers
      unless branch_changes.empty?
        # one reason is enough
        abort_auto_merge(merge_request,
                         "#{branch_changes.first} branch was changed")
        merge_request.mark_as_unchecked
      end

      handle_milestone_change(merge_request)

      added_labels = merge_request.labels - old_labels
      if added_labels.present?
        notification_service.async.relabeled_merge_request(
          merge_request,
          added_labels,
          current_user
        )
      end

      added_mentions = merge_request.mentioned_users(current_user) - old_mentioned_users

      if added_mentions.present?
        notification_service.async.new_mentions_in_merge_request(
          merge_request,
          added_mentions,
          current_user
        )
      end
    end

    def handle_task_changes(merge_request)
      todo_service.resolve_todos_for_target(merge_request, current_user)
      todo_service.update_merge_request(merge_request, current_user)
    end

    def reopen_service
      MergeRequests::ReopenService
    end

    def close_service
      MergeRequests::CloseService
    end

    def before_update(issuable, skip_spam_check: false)
      return unless issuable.changed?

      @issuable_changes = issuable.changes
    end

    def after_update(issuable)
      issuable.cache_merge_request_closes_issues!(current_user)

      return unless @issuable_changes

      %w(title description).each do |action|
        next unless @issuable_changes.key?(action)

        # Track edits to title or description
        #
        merge_request_activity_counter
          .public_send("track_#{action}_edit_action".to_sym, user: current_user) # rubocop:disable GitlabSecurity/PublicSend

        # Track changes to Draft/WIP status
        #
        if action == "title"
          old_title, new_title = @issuable_changes["title"]
          old_title_wip = MergeRequest.work_in_progress?(old_title)
          new_title_wip = MergeRequest.work_in_progress?(new_title)

          if !old_title_wip && new_title_wip
            # Marked as Draft/WIP
            #
            merge_request_activity_counter
              .track_marked_as_draft_action(user: current_user)
          elsif old_title_wip && !new_title_wip
            # Unmarked as Draft/WIP
            #
            merge_request_activity_counter
              .track_unmarked_as_draft_action(user: current_user)
          end
        end
      end
    end
    # rubocop:enable Metrics/CyclomaticComplexity
    # rubocop:enable Metrics/AbcSize

    private

    attr_reader :target_branch_was_deleted

    def handle_milestone_change(merge_request)
      return if skip_milestone_email

      return unless merge_request.previous_changes.include?('milestone_id')

      if merge_request.milestone.nil?
        notification_service.async.removed_milestone_merge_request(merge_request, current_user)
      else
        notification_service.async.changed_milestone_merge_request(merge_request, merge_request.milestone, current_user)
      end
    end

    def handle_assignees_change(merge_request, old_assignees)
      create_assignee_note(merge_request, old_assignees)
      notification_service.async.reassigned_merge_request(merge_request, current_user, old_assignees)
      todo_service.reassigned_assignable(merge_request, current_user, old_assignees)

      new_assignees = merge_request.assignees - old_assignees
      merge_request_activity_counter.track_users_assigned_to_mr(users: new_assignees)
    end

    def handle_reviewers_change(merge_request, old_reviewers)
      affected_reviewers = (old_reviewers + merge_request.reviewers) - (old_reviewers & merge_request.reviewers)
      create_reviewer_note(merge_request, old_reviewers)
      notification_service.async.changed_reviewer_of_merge_request(merge_request, current_user, old_reviewers)
      todo_service.reassigned_reviewable(merge_request, current_user, old_reviewers)
      invalidate_cache_counts(merge_request, users: affected_reviewers.compact)

      new_reviewers = merge_request.reviewers - old_reviewers
      merge_request_activity_counter.track_users_review_requested(users: new_reviewers)
    end

    def create_branch_change_note(issuable, branch_type, event_type, old_branch, new_branch)
      SystemNoteService.change_branch(
        issuable, issuable.project, current_user, branch_type, event_type,
        old_branch, new_branch)
    end

    override :handle_quick_actions
    def handle_quick_actions(merge_request)
      super

      # Ensure this parameter does not get used as an attribute
      rebase = params.delete(:rebase)

      if rebase
        rebase_from_quick_action(merge_request)
        # Ignore "/merge" if "/rebase" is used to avoid an unexpected race
        params.delete(:merge)
      end

      merge_from_quick_action(merge_request) if params[:merge]
    end

    def rebase_from_quick_action(merge_request)
      merge_request.rebase_async(current_user.id)
    end

    def merge_from_quick_action(merge_request)
      last_diff_sha = params.delete(:merge)

      MergeRequests::MergeOrchestrationService
        .new(project, current_user, { sha: last_diff_sha })
        .execute(merge_request)
    end

    override :quick_action_options
    def quick_action_options
      { merge_request_diff_head_sha: params.delete(:merge_request_diff_head_sha) }
    end
  end
end

MergeRequests::UpdateService.prepend_if_ee('EE::MergeRequests::UpdateService')
